/* ===========================================================================

	555mon.c	- Axiom Simple Monitor for the MPC555 PowerPC Micro

	Version:		2.0
    Author:         Dusty Nidey, Axiom Mfg.
	Compiler:		Diab Data Compiler version 4.2
	Clock:			4 MHz Crystal

	Initialization from SingleStep debugger file: mpc555.cfg

 (C) Copyright 2000 Axiom Manufacturing.  All rights reserved.
==============================================================================

 MEMORY MAP

0x0000 0000		CMF Internal Flash Array - CDR MoneT Flash EEPROM
0x0006 FFFF

0x0007 0000		Reserved by internal memory block
0x002F BFFF

0x002F C000		USIU - Control Registers
0x002F C3FC

0x002F C800     CMF - Control Registers
0x002F C848

0x0030 0000     DPTRAM - Control Registers
0x0030 000A

0x0030 2000		DPTRAM Dual Port TPU RAM Array
0x0030 37FF

0x0030 4000		TPU3 Time Processor Unit - Registers
0x0030 45FF

0x0030 4800		QADC64 Queued A/D Converter - Registers
0x0030 4FFF

0x0030 5000		QSMCM Queued Serial - Registers
0x0030 51DF

0x0030 6000		MIOS1 Modular I/O Subsystem - Registers
0x0030 6C70

0x0030 7080		TouCAN - Registers
0x0030 74A6

0x0030 7F80		UIMB U-Bus to IMB3 Bus Interface- Registers
0x0030 7FA0

0x0038 0000		Internal SRAM - Control Registers
0x0038 000C

0x003F 8000		Reserved
0x003F 97FF

0x003F 9800		Internal SRAM Array
0x003F FFFF

------------------------------------------------------------------------------
*/

#include "555cmf.h"
#include "diab.h"
#include "serial.h"

/*-------------------
Constant Definitions
--------------------*/

#define HWORD short
#define WORD long

#define TARGET_RAM		0
#define TARGET_EFLASH	1
#define TARGET_IFLASH	2
#define TARGET_EFLASH_O	3

/*------------
RAM Variables
------------*/
unsigned long flashaddr;	// beginning address of flash
unsigned long temp1, temp2;

void (* RamFunction)();				// this is a function pointer that holds the starting address in ram to execute


/*------------------
Function Prototypes
------------------*/


void SendPrompt();
void MainMenu();
char DoCommand();
void DoMemoryDump(char msize);
void DoMemoryModify(char msize);
void DoExecute();
void DoLoadFile(char target);
void DisplayHelp();


//---------------------------------------------------------------------------
void EPEEtest(){
    if(((get32(0x2fc808)) & 0x0020) == 0){  // if PROG_EN jumper is removed...
		RamFunction = (void (*)()) 0x10100;		// address of user program's reset vector
		RamFunction();							// go execute their code
	}
}
//---------------------------------------------------------------------------
// INITIALIZE HARDWARE REGISTERS
// This routine is called once following RESET
void inithw(){
	unsigned long *Rptr;
	unsigned short *sRptr;

	put32(0x2fc000,0x420800);	// SIUMCR make irq's general purpose i/o
	put32(0x2fc004,0x00FF88);	// SYPCR

    put32(0x2fc100,0x400003);   // BR0 32-bit
	put32(0x2fc104,0xfff80022);	// OR0 2 wait states
	put32(0x2fc108,0x800003);	// BR1
	put32(0x2fc10c,0xfff80002);	// OR1
	put32(0x2fc110,0xc00003);	// BR2
    put32(0x2fc114,0xffE00022); // OR2
    put32(0x2fc118,0x01000403); // BR3
    put32(0x2fc11c,0xfffe0FF1); // OR3
	put32(0x2fc140,0x00000000);	// DMBR
	put32(0x2fc144,0x00000000);	// DMOR
	put32(0x2fc280,0x10000);	// SCCR
	put32(0x2fc284,0x400000);	// PLPRCR 20mhz
//	put32(0x2fc284,0x900000);	// PLPRCR 40mhz
	put16(0x300000,0x00);	// DPT
	put16(0x300004,0xffa0);	// RAM base addr
	put16(0x305014,0x00);	// PORTQS
	put16(0x305016,0x00);	// PQSPAR/DDRQS
	put16(0x306100,0x00);	// MPIOSMDR
	put16(0x306102,0x00);	// MPIOSMDDR
	put16(0x306800,0x00);	// MPIOS1TPCR
	put16(0x380000,0x00);	// SRAMMCR
	put32(0x2fc024,0x00);	// SGPIODT1
	put32(0x2fc028,0x00);	// SGPIODT2
	put32(0x2fc02c,0x00);	// SGPIOCR
	put32(0x2fc030,0x00);	// EMCR
	put32(0x307f80,0x00);	// UMCR
}
// read the default serial data, ignoring the data, until it is empty
void ClearSerial(){
	do{
		ReadSerialPort(ioport);			// read the data from serial port
	}while(SerialReady(ioport) == 0);	// loop until no more serial data
}

// delay the specified number of milliseconds using the pit timer
void delay(int dtime){
	unsigned long pval, nval;
	unsigned int dloop;

	dloop = dtime * 8;	// calculate 1 ms time
	if(dtime > 2){
		dloop += (dtime / 2);	// add fraction
	}
	if(dtime > 10){
		dloop += (dtime / 10);	// add fraction
	}

	while(dloop > 0){
		for(pval=0xFF; pval > 0; pval--){
			ClearWatchdog();	// feed the dog
		}
		--dloop;
	}
}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void DoLoadFile(char target){
	int bcount, ipos, tpos;
	long new_address, off_address, off_value, off_mode;
	unsigned char *MemPtr, *tptr;
	unsigned char tbuf[8];

	off_address = flashaddr;	// save flash CS / offset address
	off_mode = 0;	// set to 0 to calculate from the first s3 record
	temp1 = 0;	// clear word programmed counter
	temp2 = 0;	// clear verify error counter

	puts("\n\n Send Motorola hex file now...");
	echo = FALSE;
	tpos=0;
	for( ; ;){
		GetString();		// receive a command string from the serial port (must end with CR)

		if((inbuf[0] != 'S') && (inbuf[0] != 's')){
			puts("\n\n ERROR: Invalid S-Record Format.\n\n");
			return;
		}
		if(inbuf[1] == '3'){
			bcount = StrHexNum(&inbuf[2], 1);	// get byte count
			bcount -=5;	// remove address and checksum from the count
			bcount *=2;	// adjust for ascii
            new_address = StrHexNum(&inbuf[4], 2);
            new_address *= 0x10000;
            new_address += StrHexNum(&inbuf[8], 2);

			MemPtr = (unsigned char *) new_address;

            for(ipos=0; ipos < bcount; ipos+=2){
                *MemPtr = StrHexNum(&inbuf[ipos+12], 1);
                ++MemPtr;
            }
		}
		putchar('.');

		if(inbuf[1] == '7' || inbuf[1] == '9'){	// end of s19 record
            StartAddr = StrHexNum(&inbuf[4], 2);
            StartAddr *= 0x10000;
            StartAddr += StrHexNum(&inbuf[8], 2);

			puts("\n\nDone.  ");
			puts("Words programmed = ");	puti(temp1);
			puts("   Verify errors = ");	puti(temp2);
			puts("\n\nUse the go command to execute.  (");
			puth(StartAddr,0);
			puts(")");
			return;
		}
	}
}

void DoExecute(){

	if(inbuf[Rcount+2] != 0){	// if there are parameters
		Rcount += 3;
		if(GetAddr()){ // get the address parameter from the string
			puts("-- NOT A VALID ADDRESS");
			return;
		}
	}
	else{
		Addr = StartAddr;	// no address specified, use the S7 Record address
	}
	RamFunction = (void (*)()) Addr;
	RamFunction();	// go execute the code in ram
}

// globals used:	unsigned long Addr, StartAddr, EndAddr;
void FillMemory(){
	unsigned char *MemPtrB;
	unsigned short *MemPtrH;
	unsigned long *MemPtrW;
	unsigned long Value;

	Rcount += 3;
	if(GetAddr()){ // get the starting address from the string
		puts("-- NOT A VALID STARTING ADDRESS");		return;
	}
	StartAddr = Addr;
	++Rcount;
	if(GetAddr()){ // get the ending address from the string
		puts("-- NOT A VALID ENDING ADDRESS");  		return;
	}
	EndAddr = Addr;
	++Rcount;
	if(GetAddr()){ // get the value to fill with from the string
		puts("-- NOT A VALID HEX VALUE");       		return;
	}
	Value = Addr;

	puts("\n\n Filling address "); puth(StartAddr,8); puts(" thru "); puth(EndAddr,8); puts(" with ");
	if(Value < 0x100){
		puth(Value,2); puts(" ...");
		MemPtrB = (unsigned char *) StartAddr;
		while(MemPtrB < (unsigned char *) EndAddr){
			*MemPtrB = (unsigned char) Value;
			++MemPtrB;
		}
	}
	else if(Value < 0x10000){
		puth(Value,4); puts(" ...");
		MemPtrH = (unsigned short *) StartAddr;
		while(MemPtrH < (unsigned short *) EndAddr){
			*MemPtrH = (unsigned short) Value;
			++MemPtrH;
		}
	}
	else{
		puth(Value,8); puts(" ...");
		MemPtrW = (unsigned long *) StartAddr;
		while(MemPtrW < (unsigned long *) EndAddr){
			*MemPtrW = (unsigned long) Value;
			++MemPtrW;
		}
	}

	puts(" Done.\n");
}

void DoMemoryModify(char msize){
	unsigned char *MemPtrB;
	unsigned short *MemPtrH;
	unsigned long *MemPtrW;

	if(inbuf[Rcount+2] != 0){	// if there are parameters
		if(inbuf[Rcount+2] != ' ')	++Rcount;	// allow for extra character (b,h or w)
		Rcount += 3;
		if(GetAddr()){ // get the address parameter from the string
			puts("-- NOT A VALID ADDRESS");
			return;
		}
	}
	// display address entered followed by it's current value
	puts("\n");

	puth(Addr,8);
	puts(":  ");

	switch(msize){
		case 'b':
			MemPtrB = (unsigned char *) Addr;
			puth(*MemPtrB,2);
			break;
		case 'h':
			MemPtrH = (unsigned short *) Addr;
			puth(*MemPtrH,4);
			break;
		default:
			MemPtrW = (unsigned long *) Addr;
			puth(*MemPtrW,8);
			break;
	}

	puts("  ");

	// now wait for user to enter a new value
	GetString();		// receive a command string from the serial port (must end with CR)

	if(GetAddr()){ // get the new byte value, disregard if not a valid number
		return;
	}

	switch(msize){
		case 'b':
			*MemPtrB = (unsigned char) Addr;
			break;
		case 'h':
			*MemPtrH = (unsigned short) Addr;
			break;
		default:
			*MemPtrW = (unsigned long) Addr;
			break;
	}

}

void DoMemoryDump(char msize){
	char lcount;
	short ccount;
	unsigned char *MemPtrB;
	unsigned short *MemPtrH;
	unsigned long *MemPtrW;


	EndAddr = 0;
	if(inbuf[Rcount+2] != 0){	// if there are parameters
		if(inbuf[Rcount+2] != ' ')	++Rcount;	// allow for extra character (b,h or w)
		Rcount += 3;
		if(GetAddr()){ // get the starting address from the string
			puts("-- NOT A VALID STARTING ADDRESS");
			return;
		}
		StartAddr = Addr;
		if(inbuf[Rcount] == ' '){
			++Rcount;
			if(GetAddr() == 0){ // get the ending address from the string
				EndAddr = Addr;
			}
		}
	}

	Addr = StartAddr;
	// display the memory formatted in hex strings, starting with StartAddr
	// ends with StartAddr pointing to the next block in line to be displayed
	if(msize == 'b')		puts("\n          0  1  2  3  4  5  6  7   8  9  a  b  c  d  e  f");
	else if(msize == 'h')	puts("\n          0 1  2 3  4 5  6 7   8 9  a b  c d  e f");
	else                 	puts("\n          0 1 2 3  4 5 6 7   8 9 a b  c d e f");

	for(lcount=0; lcount < 10; lcount++){
		puts("\n");
		puth(Addr,8);
		puts("  ");
		switch(msize){
			case 'b':
				MemPtrB = (unsigned char *) Addr;
				for(ccount=0; ccount < 16; ccount++){
					puth(*MemPtrB,2);
					if(ccount == 7)	puts("  ");
					else			putchar(' ');
					++MemPtrB;
				}
				Addr += 16;
				break;
			case 'h':
				MemPtrH = (unsigned short *) Addr;
				for(ccount=0; ccount < 8; ccount++){
					puth(*MemPtrH,4);
					if(ccount == 3)	puts("  ");
					else			putchar(' ');
					++MemPtrH;  // compiler is doing a left shift (+2) here
				}
				Addr += 16;
				break;
			default:
				MemPtrW = (unsigned long *) Addr;
				for(ccount=0; ccount < 4; ccount++){
					puth(*MemPtrW,8);
					if(ccount == 1)	puts("  ");
					else			putchar(' ');
					++MemPtrW;	// compiler is doing a +4 here
				}
				Addr += 16;
				break;
		}
	}

}

//----------------------------------------------------------------------------
void DisplayHelp(){
	puts("\n\n mm[bhw] [address]           Modify a Byte, Half Word or Word at [address]");
	puts("\n                                    Example:  mmw 3fc000");
	puts("\n md[bhw] [address]           View Memory as Byte, H-Word or Word at [address]");
	puts("\n                                    mdh 800000");
	puts("\n bf [start] [end] [value]    Fill Memory from [start] to [end] with [value]");
	puts("\n                                    bf 800000 810000 FFFF");
	puts("\n dl                          Download a hex file into memory      dl");
	puts("\n go [address]                Go execute a program at [address]    go 6000000");
	puts("\n\n exit                        Return to Main Menu                  ex");
}

//----------------------------------------------------------------------------
char DoCommand(){
	char byte1,byte2,byte3;

	byte1 = inbuf[Rcount];
	byte2 = inbuf[Rcount+1];
	byte3 = inbuf[Rcount+2];

	if(byte1 == 'e'){
		if(byte2 == 'x')	return 1;
	}
	else if(byte1 == 'm'){
		if(byte2 == 'd'){
			if(byte3 == 'b')		DoMemoryDump('b');
			else if(byte3 == 'h')	DoMemoryDump('h');
			else					DoMemoryDump('w');
		}
		if(byte2 == 'm'){
			if(byte3 == 'b')		DoMemoryModify('b');
			else if(byte3 == 'h')	DoMemoryModify('h');
			else					DoMemoryModify('w');
		}
	}
	else if(byte1 == 'b'){
		FillMemory();
	}
	else if(byte1 == 'd'){
		if(byte2 == 'l')	DoLoadFile(TARGET_RAM);
	}
	else if(byte1 == 'g'){
		if(byte2 == 'o')	DoExecute();
	}
	else if(byte1 == 'h'){
		DisplayHelp();
	}
	else{
		puts("\n --- Unrecognised Command.  Type HELP for a list of commands.\n\n");
	}
	return 0;
}


void SendPrompt(){
	puts("\n\n>");
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
/*----------------------------------------------------------------------------
main
----
 Main program loop.  This is called after the hardware initialization and
 reset code.  The external serial ports are initialized and then an endless
 loop begins which simply outputs menus to the serial port as text strings
 followed by a prompt for user input thru the same serial port.

 The 3 main utilities accessable to the user thru these menus include:
	- Testing functions
	- Debug monitor
	- Programming utilities for the onboard flash
----------------------------------------------------------------------------*/

void main ( void ){
	UNS8 choice;

	InitCom(0);	// initialize SCI1 serial port 1
	InitCom(1);	// initialize SCI2 serial port 2
	ioport = 0;	// default to 0 for now, don't detect.

	for( ; ;){
		echo = FALSE;
		puts("\n\n    ---------------------------------");
        puts("\n    AXIOM PB-555 UTILITIES        1.0");
		puts("\n    ---------------------------------");
		puts("\n \n 1. Debug Monitor");
		puts("\n 2. On-Chip CMF Flash Programming");
		puts("\n\n Select: ");
		choice = tolower(GetSByte());
		puts("\n\n");
		switch(choice){
			case '1':
                puts("\n --------------------------------------");
                puts("\n Axiom MPC555 Simple Monitor - Rev C.3\n");
                puts("\n Type  HELP  for a list of commands");
                puts("\n Type  EXIT  to return to the menu");
                do{
                    SendPrompt();
                    echo = TRUE;
                    GetString();        // receive a command string from the serial port (must end with CR)
                }while(DoCommand() == 0);
				break;

			case '2':
                CMF_Menu();
				break;
		}

	}
}
